@@ -548,14 +548,14 @@ bandwidth.pop <- function(data,UD,kernel="individual",weights=FALSE,ref="Gaussia
     {
       for(j in (i+1)%:%length(data))
       {
-        MU <- c( CTMM[[i]]$mu - CTMM[[j]]$mu )
+        MU <- c( CTMM[[i]]$mu[1,] - CTMM[[j]]$mu[1,] )
 
         if(kernel=="individual")
         { SIG <- (1+H)*(methods::getDataPart(CTMM[[i]]$sigma) + methods::getDataPart(CTMM[[j]]$sigma)) }
         else if(kernel=="population")
         { SIG <- methods::getDataPart(CTMM[[i]]$sigma) + methods::getDataPart(CTMM[[j]]$sigma) + 2*H.M }
 
-        Q[i,j] <- Q[j,i] <- exp(-(MU %*% PDsolve(SIG) %*% MU)/2) / sqrt(det(SIG))
+        Q[i,j] <- Q[j,i] <- exp(-(MU %*% pd.solve(SIG) %*% MU)/2) / sqrt(det(SIG))
       }
     }
 
@@ -564,14 +564,14 @@ bandwidth.pop <- function(data,UD,kernel="individual",weights=FALSE,ref="Gaussia
     L <- rep(0,length(data))
     for(i in 1:length(data))
     {
-      MU <- c( CTMM[[i]]$mu - MEAN$mu )
+      MU <- c( CTMM[[i]]$mu[1,] - MEAN$mu[1,] )
 
       if(kernel=="individual")
       { SIG <- methods::getDataPart(MEAN$sigma) + (1+H)*methods::getDataPart(CTMM[[i]]$sigma) }
       else if(kernel=="population")
       { SIG <- methods::getDataPart(MEAN$sigma) + methods::getDataPart(CTMM[[i]]$sigma) + H.M }
 
-      L[i] <- exp(-(MU %*% PDsolve(SIG) %*% MU)/2) / sqrt(det(SIG))
+      L[i] <- exp(-(MU %*% pd.solve(SIG) %*% MU)/2) / sqrt(det(SIG))
     }
 
     # correct non-Gaussian overlap
@@ -589,7 +589,7 @@ bandwidth.pop <- function(data,UD,kernel="individual",weights=FALSE,ref="Gaussia
       # # w = solve(Q).(L+lambda*1)
       # # 1 = 1.solve(Q).L + 1.solve(Q).1 * lambda
       # # lambda = (1-1.solve(Q).L)/(1.solve(Q).1)
-      # iQ <- PDsolve(Q)
+      # iQ <- pd.solve(Q)
       # lambda <- (1-sum(iQ%*%L))/sum(iQ)
       # w <- c( iQ %*% (L+lambda) )
 
@@ -655,7 +655,7 @@ bandwidth.pop <- function(data,UD,kernel="individual",weights=FALSE,ref="Gaussia
   M1 <- M2 <- 0
   for(i in 1:length(UD))
   {
-    MU <- c( CTMM[[i]]$mu - MEAN$mu )
+    MU <- c( CTMM[[i]]$mu[1,] - MEAN$mu[1,] )
     M1 <- M1 + weights[i] * MU
     if(kernel=="individual")
     { M2 <- M2 + weights[i] * ( UD[[i]]$COV + H*CTMM[[i]]$sigma + outer(MU) ) }
