@@ -10,8 +10,7 @@ smoother <- function(data,CTMM,precompute=FALSE,sample=FALSE,residual=FALSE,...)
   AXES <- length(CTMM$axes)
 
   t <- data$t
-
-  dt <- c(Inf,diff(t))
+  dt <- CTMM$dt
   n <- length(t)
 
   isotropic <- CTMM$isotropic
@@ -133,10 +132,10 @@ smoother <- function(data,CTMM,precompute=FALSE,sample=FALSE,residual=FALSE,...)
   {
     # major axis likelihood
     CTMM$sigma <- SIGMA[1]
-    KALMAN1 <- kalman(z[,1,drop=FALSE],u=NULL,t=t,dt=dt,CTMM=CTMM,error=error[,1,1,drop=FALSE],precompute=precompute,sample=sample,residual=residual,...)
+    KALMAN1 <- kalman(z[,1,drop=FALSE],u=NULL,t=t,CTMM=CTMM,error=error[,1,1,drop=FALSE],precompute=precompute,sample=sample,residual=residual,...)
     # minor axis likelihood
     CTMM$sigma <- SIGMA[2]
-    KALMAN2 <- kalman(z[,2,drop=FALSE],u=NULL,t=t,dt=dt,CTMM=CTMM,error=error[,2,2,drop=FALSE],precompute=precompute,sample=sample,residual=residual,...)
+    KALMAN2 <- kalman(z[,2,drop=FALSE],u=NULL,t=t,CTMM=CTMM,error=error[,2,2,drop=FALSE],precompute=precompute,sample=sample,residual=residual,...)
 
     if(residual) { return(cbind(KALMAN1,KALMAN2)) }
 
@@ -166,7 +165,7 @@ smoother <- function(data,CTMM,precompute=FALSE,sample=FALSE,residual=FALSE,...)
       error <- error[,1,1,drop=FALSE] # isotropic && UERE redundant error information
     }
 
-    KALMAN <- kalman(z,u=NULL,t=t,dt=dt,CTMM=CTMM,error=error,DIM=DIM,precompute=precompute,sample=sample,residual=residual,...)
+    KALMAN <- kalman(z,u=NULL,t=t,CTMM=CTMM,error=error,DIM=DIM,precompute=precompute,sample=sample,residual=residual,...)
     # point estimates will be correct but eccentricity is missing from variances
 
     if(residual) { return(KALMAN) }
@@ -374,11 +373,7 @@ simulate.ctmm <- function(object,nsim=1,seed=NULL,data=NULL,VMM=NULL,t=NULL,dt=N
 {
   T.SPECIFIED <- !is.null(t)
 
-  info <- attr(object,"info")
-  if(!is.null(data)) { info$identity <- glue( attr(data,'info')$identity , info$identity ) }
-
-  # have to do this becaues simulate is an S3 with 3 fixed arguments
-
+  # I have to do this because simulate is an S3 method with 3 fixed arguments
   if(class(object)[1] %in% c("data.frame","telemetry"))
   {
     TEMP <- data
@@ -414,6 +409,9 @@ simulate.ctmm <- function(object,nsim=1,seed=NULL,data=NULL,VMM=NULL,t=NULL,dt=N
     seed <- TEMP
   }
 
+  info <- attr(object,"info")
+  if(!is.null(data)) { info$identity <- glue( attr(data,'info')$identity , info$identity ) }
+
   if(is.null(nsim)) { nsim <- 1 }
 
   if(nsim>1)
@@ -471,14 +469,14 @@ simulate.ctmm <- function(object,nsim=1,seed=NULL,data=NULL,VMM=NULL,t=NULL,dt=N
     STUFF <- c('object','data','drift','velocity')
     if(precompute>=0) # prepare model and data frame
     {
-      object <- ctmm.prepare(data,object,precompute=FALSE) # u calculated here with unfilled t
+      object <- ctmm.prepare(data,object,precompute=FALSE,dt=FALSE) # u calculated here with unfilled t - why did I need this?
       data <- fill.data(data,CTMM=object,t=t,dt=dt,res=res,...)
+      object <- ctmm.prepare(data,object,precompute=FALSE) # u calculated here with filled t
       # object$error <- TRUE # avoids unit variance algorithm - data contains fixed errors from fill.data
 
       # calculate trend
-      drift <- get(object$mean)
-      velocity <- drift@velocity(data$t,object) %*% object$mu
-      drift <- drift(data$t,object) %*% object$mu
+      velocity <- drift.velocity(object,data$t) %*% object$mu
+      drift <- drift.mean(object,data$t) %*% object$mu
 
       # detrend for simulation - retrend later
       z <- get.telemetry(data,axes=object$axes)
@@ -511,6 +509,18 @@ simulate.ctmm <- function(object,nsim=1,seed=NULL,data=NULL,VMM=NULL,t=NULL,dt=N
   } # conditional simulation
   else # Gaussian simulation not conditioned off of any data
   {
+    # POPULATION SIMULATION
+    if(length(object$isotropic)>1)
+    {
+      object$COV.mu <- object$POV.mu
+      object$COV <- object$POV
+      object$POV.mu <- object$POV <- NULL
+      object$isotropic <- object$isotropic["sigma"]
+      object <- emulate(object,fast=TRUE)
+      return(object)
+    }
+
+    # INDIVIDUAL SIMULATION
     STUFF <- c('Green','Sigma','error','ELLIPSE','object','mu','Lambda','n','K','z','v','circle','R')
     if(precompute>=0)
     {
@@ -546,7 +556,10 @@ simulate.ctmm <- function(object,nsim=1,seed=NULL,data=NULL,VMM=NULL,t=NULL,dt=N
 
         K <- length(tau)
 
-        dt <- c(Inf,diff(t)) # time lags
+        # time lags information from ctmm.prepare
+        dt <- object$dt
+        dti <- object$dti
+        dtl <- object$dtl
 
         # where we will store the data
         z <- array(0,c(n,AXES))
@@ -557,13 +570,23 @@ simulate.ctmm <- function(object,nsim=1,seed=NULL,data=NULL,VMM=NULL,t=NULL,dt=N
 
         object$sigma <- 1
         object <- get.taus(object) # pre-compute stuff for Langevin equation solutions
-        for(i in 1:n)
+
+        j <- 1 # sorted index
+        for(i in 1:length(dtl)) # level index
         {
-          # tabulate propagators if necessary
-          if((i==1)||(dt[i]!=dt[i-1])) { Langevin <- langevin(dt=dt[i],CTMM=object) }
-          Green[i,,] <- Langevin$Green
-          Sigma[i,,] <- Langevin$Sigma
-        }
+          LANGEVIN <- langevin(dt=dtl[i],CTMM=object)
+
+          k <- dti[j] # time index
+          while(dt[k]==dtl[i])
+          {
+            Green[k,,] <- LANGEVIN$Green # (K*DIM,K*DIM)
+            Sigma[k,,] <- LANGEVIN$Sigma # (K*DIM,K*DIM)
+
+            j <- j + 1 # sorted index
+            if(j>length(dti)) { break }
+            k <- dti[j] # time index
+          } # end same time-lags
+        } # end all time-lag levels
         if(!object$range) { Sigma[1,,] <- 0 } # start at first point instead of random point on Earth
 
         # Sigma is now standardization matrix
@@ -625,7 +648,7 @@ simulate.ctmm <- function(object,nsim=1,seed=NULL,data=NULL,VMM=NULL,t=NULL,dt=N
       colnames(z) <- axes
       if(K>1)
       {
-        v <- (v %*% Lambda) + (get(object$mean)@velocity(t,object) %*% mu)
+        v <- (v %*% Lambda) + (drift.velocity(object,t) %*% mu)
         colnames(v) <- paste0("v",axes)
       }
     } # end if(!is.null(object))
@@ -727,7 +750,7 @@ predict.ctmm <- function(object,data=NULL,VMM=NULL,t=NULL,dt=NULL,res=1,complete
     r <- object$mean.vec %*% mu
     colnames(r) <- axes
 
-    v <- get(object$mean)@velocity(t,object) %*% mu
+    v <- drift.velocity(object,t) %*% mu
     colnames(v) <- paste0("v",axes)
 
     data <- data.frame(r,v)
@@ -760,15 +783,14 @@ predict.ctmm <- function(object,data=NULL,VMM=NULL,t=NULL,dt=NULL,res=1,complete
   }
   else # condition off of the data
   {
-    # object <- ctmm.prepare(data,object) # mean.vec here is calculated with pre-filled t
     K <- length(object$tau)
     data <- fill.data(data,CTMM=object,t=t,dt=dt,res=res)
+    object <- ctmm.prepare(data,object) # is there any reason not to do this here?
     # object$error <- TRUE # avoids unit variance algorithm
 
     # calculate trend
-    drift <- get(object$mean)
-    velocity <- drift@velocity(data$t,object) %*% object$mu
-    drift <- drift(data$t,object) %*% object$mu
+    velocity <- drift.velocity(object,data$t) %*% object$mu
+    drift <- drift.mean(object,data$t) %*% object$mu
 
     # detrend for simulation - retrend later
     z <- get.telemetry(data,axes=axes)
