@@ -1,7 +1,54 @@
-# drift/mean function models
-# guess some parameters and check the model parameter sanity
-drift.init <- function(data,CTMM)
+# define all generic drift functions
+drift.fn <- function(fn,CTMM,...)
 {
+  F1 <- paste0(CTMM$mean,".",fn) # preferred function
+  F2 <- paste0("stationary.",fn) # default function
+
+  # mget is buggy
+  # fn <- mget(F1, mode="function", ifnotfound=list(F2=get(F2,mode="function")), inherits=TRUE)[[1]]
+
+  if(exists(F1,mode="function"))
+  { fn <- get(F1,mode="function") }
+  else
+  { fn <- get(F2,mode="function") }
+
+  fn(CTMM,...)
+}
+
+# could not figure out how to automate assign here
+drift.name <- function(CTMM,...) { drift.fn("name",CTMM,...) }
+drift.pars <- function(CTMM,...) { drift.fn("pars",CTMM,...) }
+drift.assign <- function(CTMM,...) { drift.fn("assign",CTMM,...) }
+drift.mean <- function(CTMM,...) { drift.fn("mean",CTMM,...) }
+drift.velocity <- function(CTMM,...) { drift.fn("velocity",CTMM,...) }
+drift.energy <- function(CTMM,...) { drift.fn("energy",CTMM,...) }
+drift.init <- function(CTMM,...) { drift.fn("init",CTMM,...) }
+drift.shift <- function(CTMM,...) { drift.fn("shift",CTMM,...) }
+drift.svf <- function(CTMM,...) { drift.fn("svf",CTMM,...) }
+drift.complexify <- function(CTMM,...) { drift.fn("complexify",CTMM,...) }
+drift.simplify <- function(CTMM,...) { drift.fn("simplify",CTMM,...) }
+drift.is.stationary <- function(CTMM,...) { drift.fn("is.stationary",CTMM,...) }
+drift.speed <- function(CTMM,...) { drift.fn("speed",CTMM,...) }
+drift.summary <- function(CTMM,...) { drift.fn("summary",CTMM,...) }
+drift.scale <- function(CTMM,...) { drift.fn("scale",CTMM,...) }
+drift.is.finite <- function(CTMM,...) { drift.fn("is.finite",CTMM,...) }
+
+###################################
+# stationary and mean-zero functions
+
+# name of mean function
+stationary.name <- function(CTMM,...) { NULL }
+
+zero.name <- function(CTMM,...) { "mean-zero" }
+
+# is this mean function stationary?
+stationary.is.stationary <- function(CTMM,...) { TRUE }
+
+# initialization for stationary (and similar structure) mean functions
+stationary.init <- function(CTMM,data=NULL,...)
+{
+  if(!is.null(CTMM$mu) && !is.null(CTMM$sigma)) { return(CTMM) }
+
   AXES <- length(CTMM$axes)
   z <- get.telemetry(data,CTMM$axes)
 
@@ -16,13 +63,14 @@ drift.init <- function(data,CTMM)
   # normalize weights
   # w <- w/sum(w)
 
-  drift <- get(CTMM$mean)
-  u <- drift(data$t,CTMM)
+  u <- drift.mean(CTMM,data$t,...)
   # IID estimate for (error + 0 movement) || (movement + 0 error)
-  CTMM$mu <- PDsolve(t(u) %*% (w * u)) %*% (t(u) %*% (w * z))
+
+  if(is.null(CTMM$mu))
+  { CTMM$mu <- pd.solve(t(u) %*% (w * u)) %*% (t(u) %*% (w * z)) }
 
   # don't return variance if no variance
-  if(!CTMM$range) { return(CTMM) }
+  if(!CTMM$range || !is.null(CTMM$sigma)) { return(CTMM) }
 
   n <- length(data$t)
   z <- z - (u %*% CTMM$mu)
@@ -45,84 +93,45 @@ drift.init <- function(data,CTMM)
   return(CTMM)
 }
 
+# x(t) = mu * u(t) + e(t)
+stationary.mean <- function(CTMM,t,...) { cbind( array(1,length(t)) ) }
+
+zero.mean <- function(CTMM,t,...) { cbind( array(0,length(t)) ) }
+
+# v(t) = mu * u'(t) + e'(t)
+stationary.velocity <- function(CTMM,t,...) { cbind( array(0,length(t)) ) }
+
+# non-linear parameters
+stationary.assign <- function(CTMM,value) { CTMM }
+stationary.pars <- function(CTMM,all=FALSE,...) { if(all) { list() } else { NULL } }
+
+# build-up / break-down mean function
+stationary.simplify <- function(CTMM,...) { list() }
+stationary.complexify <- function(CTMM,...) { list() }
+
 # how do we rescale time
-drift.scale <- function(CTMM,time) { CTMM }
+stationary.scale <- function(CTMM,time,...) { CTMM }
 
 # how do we shift the mean by a constant location
 # this assumes the first mean term is always the stationary mean value and all others are deviations
-stationary.shift <- function(mu,dmu)
+stationary.shift <- function(CTMM,dmu,...)
 {
-  mu[1,] <- mu[1,] + dmu
-  return(mu)
+  CTMM$mu[1,] <- CTMM$mu[1,] + dmu
+  return(CTMM)
 }
 
 # for when all mean coefficients are centered on the origin and must be shifted
-uniform.shift <- function(mu,dmu)
+uniform.shift <- function(CTMM,dmu,...)
 {
-  mu <- t(t(mu) + dmu)
-  return(mu)
+  CTMM$mu <- t(t(CTMM$mu) + dmu)
+  return(CTMM)
 }
 
-# increase number of linear model parameters
-drift.refine <- function(CTMM) { list() }
-
-# name of mean function
-drift.name <- function(CTMM) { NULL }
-
-# place to put optional summary information
-drift.summary <- function(CTMM,level,level.UD) { NULL }
-
-prototype.drift <- function(t,CTMM) { rep(1,length(t)) }
-attr(prototype.drift,"init") <- drift.init
-attr(prototype.drift,"name") <- drift.name
-attr(prototype.drift,"refine") <- drift.refine
-attr(prototype.drift,"scale") <- drift.scale
-attr(prototype.drift,"shift") <- stationary.shift
-attr(prototype.drift,"summary") <- drift.summary
-attr(prototype.drift,"velocity") <- function(t,CTMM) { rep(0,length(t)) }
-
-new.drift <- methods::setClass("drift",
-              representation("function",
-                             energy="function",
-                             init="function",
-                             is.stationary="function",
-                             name="function",
-                             refine="function",
-                             scale="function",
-                             shift="function",
-                             speed="function",
-                             summary="function",
-                             svf="function",
-                             velocity="function"),
-              prototype=prototype.drift)
-
-########################################
-# !!! TODO !!! REDO EVERYTHING LIKE THIS
-
-drift.complexify <- function(CTMM)
-{
-  if(CTMM$mean=="stationary") { return(list()) }
-  fn <- get(paste0(CTMM$mean,".drift.complexify"))
-  fn(CTMM)
-}
-
-drift.simplify <- function(CTMM)
-{
-  if(CTMM$mean=="stationary") { return(list()) }
-  fn <- get(paste0(CTMM$mean,".drift.simplify"))
-  fn(CTMM)
-}
-
-#################################
-# stationary mean/drift function
-#################################
-stationary.drift <- function(t,CTMM) { cbind( array(1,length(t)) ) }
-stationary.velocity <- function(t,CTMM) { cbind( array(0,length(t)) ) }
-
-stationary.is.stationary <- function(CTMM) { TRUE }
+# place to put optional summary information on linear mean parameters
+stationary.summary <- function(CTMM,level,level.UD,...) { NULL }
 
 # svf of mean function
-stationary.svf <- function(CTMM)
+stationary.svf <- function(CTMM,...)
 {
   # default
   EST <- function(t) { 0 }
@@ -132,79 +141,33 @@ stationary.svf <- function(CTMM)
 }
 
 # mean square speed function: point estimate and variance
-stationary.speed <- function(CTMM) { list(EST=0,VAR=0) }
+stationary.speed <- function(CTMM,...) { list(EST=0,VAR=0) }
 
 # <t(U)%*%(U)> and <t(U')%*%(U')>
-stationary.energy <- function(CTMM) { list(UU=1,VV=0) }
+stationary.energy <- function(CTMM,...) { list(UU=1,VV=0) }
 
-# combine this all together for convenience
-stationary <- new.drift(stationary.drift,
-                        is.stationary=stationary.is.stationary,
-                        energy=stationary.energy,
-                        speed=stationary.speed,
-                        svf=stationary.svf,
-                        velocity=stationary.velocity)
+zero.energy <- function(CTMM,...) { list(UU=0,VV=0) }
 
-############################
-# mean zero process
-############################
-
-zero.drift <- function(t,CTMM) { cbind( array(0,length(t)) ) }
-
-zero.energy <- function(CTMM) { list(UU=1,VV=0) }
-
-zero <- new.drift(zero.drift,
-                  is.stationary=stationary.is.stationary,
-                  energy=zero.energy,
-                  speed=stationary.speed,
-                  svf=stationary.svf,
-                  velocity=stationary.velocity)
+# are the non-linear parameters finite?
+stationary.is.finite <- function(CTMM,...) { TRUE }
 
 ############################
 # Periodic drift function
 ############################
-periodic.is.stationary <- function(CTMM) { !sum(CTMM$harmonic) }
 
-# periodic mean/drift function
-periodic.drift <- function(t,CTMM)
+# name of mean function
+periodic.name <- function(CTMM,...)
 {
-  harmonic <- CTMM$harmonic
-  period <- CTMM$period
-
-  # constant term
-  U <- stationary.drift(t,CTMM)
-
-  omega <- periodic.omega(CTMM)
-  if(length(omega))
-  {
-    omega <- t %o% omega
-    U <- cbind( U , cos(omega) , sin(omega) )
-  }
-
-  return(U)
+  NAME <- CTMM$harmonic
+  NAME <- paste(NAME,collapse=" ")
+  NAME <- paste("harmonic",NAME)
+  return(NAME)
 }
 
-# periodic velocity mean
-periodic.velocity <- function(t,CTMM)
-{
-  harmonic <- CTMM$harmonic
-  period <- CTMM$period
-
-  # constant term
-  U <- stationary.velocity(t,CTMM)
-
-  omega <- periodic.omega(CTMM)
-  if(length(omega))
-  {
-    theta <- omega %o% t # backwards for multiplication by first dimension
-    U <- cbind( U , -t(omega*sin(theta)) , t(omega*cos(theta)) )
-  }
-
-  return(U)
-}
+periodic.is.stationary <- function(CTMM,...) { !sum(CTMM$harmonic) }
 
 # guess parameters
-periodic.init <- function(data,CTMM)
+periodic.init <- function(CTMM,data=NULL,...)
 {
   # default period of 1 day
   if(is.null(CTMM$period)) { CTMM$period <- 1 %#% "day" }
@@ -212,23 +175,229 @@ periodic.init <- function(data,CTMM)
   # default harmonics is none
   if(is.null(CTMM$harmonic)) { CTMM$harmonic <- numeric(length(CTMM$period)) }
 
-  if(is.null(CTMM$mu)) { CTMM <- drift.init(data,CTMM) }
+  if(is.null(CTMM$mu)) { CTMM <- stationary.init(CTMM,data,...) }
+
+  # in case of existing mean parameters
+  n <- (nrow(CTMM$mu)-1)/2
+  CTMM$harmonic <- pmax(CTMM$harmonic,floor(n/length(CTMM$harmonic)))
 
   # !!! maybe run generic drift here
 
   return(CTMM)
 }
 
-# how do we rescale time
-periodic.scale <- function(CTMM,time)
+
+# come up with generic name of mean vector components
+periodic.namer <- function(CTMM)
 {
+  NAME <- 1%:%sum(CTMM$harmonic)
+  if(length(NAME))
+  { NAME <- c(rbind( paste0("Sin.",NAME) , paste0("Cos.",NAME) )) }
+  NAME <- c("1",NAME)
+  return(NAME)
+}
+
+# periodic location mean
+periodic.mean <- function(CTMM,t,verbose=TRUE,...)
+{
+  harmonic <- CTMM$harmonic
+  period <- CTMM$period
+
+  # constant term
+  U <- stationary.mean(CTMM,t,...)
+
+  omega <- periodic.omega(CTMM)
+  if(length(omega))
+  {
+    Omega <- nant(t %o% omega,0)
+    U <- cbind( U , riffle(sin(Omega),cos(Omega)) )
+  }
+
+  colnames(U) <- periodic.namer(CTMM)
+
+  # short periods cause colinearity, replace with Taylor series
+  dt <- last(t)-first(t)
+  BAD <- omega*dt < 1/10
+  if(!verbose && any(BAD))
+  {
+    BAD <- viffle(BAD,BAD)
+    t <- omega*t
+    BAD <- which(BAD)
+    U[,1+BAD] <- outer(t,1:length(BAD),"^")
+  }
+
+  return(U)
+}
+
+# periodic velocity mean
+periodic.velocity <- function(CTMM,t,...)
+{
+  harmonic <- CTMM$harmonic
+  period <- CTMM$period
+
+  # constant term
+  U <- stationary.velocity(CTMM,t,...)
+
+  omega <- periodic.omega(CTMM)
+  if(length(omega))
+  {
+    theta <- omega %o% t # backwards for multiplication by first dimension
+    U <- cbind( U , riffle( t(omega*cos(theta)) , -t(omega*sin(theta)) ) )
+  }
+
+  colnames(U) <- periodic.namer(CTMM)
+
+  return(U)
+}
+
+# non-linear parameters - periods
+periodic.assign <- function(x,value)
+{
+  x$period <- 2*pi/value # frequency
+
+  # remove zero frequencies as stationary
+  ZERO <- (value==0)
+  x$period <- x$period[!ZERO]
+
+  return(x)
+}
+
+periodic.pars <- function(CTMM,all=FALSE,fit=FALSE,...)
+{
+  # fitting with no harmonics yields no new parameters
+  if(fit && periodic.is.stationary(CTMM)) { return(stationary.pars(CTMM,all=all,...)) }
+
+  CTMM$period <- 2*pi/sort(CTMM$period) # frequency
+  N <- length(CTMM$period)
+  if(N>0) { names(CTMM$period) <- paste0("period.",1:N) }
+
+  if(all)
+  {
+    R <- list()
+    R$NAMES <- names(CTMM$period)
+    R$pars <- CTMM$period
+    R$parscale <- R$pars
+    R$lower <- array(0,N)
+    R$upper <- array(Inf,N)
+  }
+  else
+  { R <- CTMM$period }
+
+  return(R)
+}
+
+# reduce number of harmonics in model
+periodic.simplify <- function(CTMM,...)
+{
+  period <- CTMM$period
+  harmonic <- CTMM$harmonic
+
+  GUESS <- list()
+
+  for(i in 1:length(period))
+  {
+    if(harmonic[i]>0)
+    {
+      GUESS[[length(GUESS)+1]] <- CTMM
+      GUESS[[length(GUESS)]]$harmonic[i] <- harmonic[i] - 1
+    }
+  }
+
+  return(GUESS)
+}
+
+# increase number of harmonics in model
+periodic.complexify <- function(CTMM,...)
+{
+  period <- CTMM$period
+  harmonic <- CTMM$harmonic
+
+  GUESS <- list()
+
+  # dt <- stats::median(diff(data$t))
+  # P <- attr(CTMM$mean,"par")$P
+  # # max harmonics for Nyquist frequency
+  # kmax <- P/(2*dt)
+
+  for(i in 1:length(period))
+  {
+    GUESS[[length(GUESS)+1]] <- CTMM
+    GUESS[[length(GUESS)]]$harmonic[i] <- harmonic[i] + 1
+  }
+
+  return(GUESS)
+}
+
+# how do we rescale time
+periodic.scale <- function(CTMM,time,...)
+{
+  # this is the period (user friendly)
   CTMM$period <- CTMM$period / time
 
+  # but this is the frequency (optimization friendly)
+  if("COV" %in% names(CTMM))
+  {
+    P <- names(CTMM$period)
+    P <- P[P %in% rownames(CTMM$COV)]
+    CTMM$COV[P,] <- CTMM$COV[P,] * time
+    CTMM$COV[,P] <- CTMM$COV[,P] * time
+  }
+
   return(CTMM)
 }
 
+# calculate rotational indices
+periodic.summary <- function(CTMM,level=0.95,level.UD=0.95,units=TRUE,...)
+{
+  alpha <- 1-level
+  NAMES <- SUM <- NULL
+
+  if(all(CTMM$harmonic==0)) { return(SUM) }
+
+  # periods
+  period <- CTMM$period # this is the period (user friendly)
+  VAR <- diag(CTMM$COV)[names(period)] # but this is the frequency (optimization friendly)
+  VAR <- VAR * period^4 / (2*pi)^2
+  for(i in 1:length(period))
+  {
+    CI <- rbind(ci.tau(period[i],VAR[i],alpha=alpha) )
+    UNITS <- unit(CI,"time",SI=!units)
+    CI <- CI/UNITS$scale
+    rownames(CI) <- paste0(names(period)[i]," (",UNITS$name,")")
+    SUM <- rbind(SUM,CI)
+  }
+
+  STUFF <- periodic.variances(CTMM)
+  R <- STUFF$R
+  R$MLE -> MLE
+  R$VAR -> VAR
+
+  CI <- beta.ci(MLE,VAR,alpha=alpha)
+  CI <- 100*sqrt(rbind(CI))
+  rownames(CI) <- "rotation/deviation %"
+
+  SUM <- rbind(SUM,CI)
+
+  # ROTATIONAL SPEED INDEX
+  # CIs are too big to be useful
+  if(length(CTMM$tau)>1 && CTMM$tau[2])
+  {
+    V <- STUFF$V
+    V$MLE -> MLE
+    V$VAR -> VAR
+
+    CI <- beta.ci(MLE,VAR,alpha=alpha)
+    CI <- 100*sqrt(rbind(CI))
+    rownames(CI) <- "rotation/speed %"
+
+    SUM <- rbind(SUM,CI)
+  }
+
+  return(SUM)
+}
+
 # SVF of mean function
-periodic.svf <- function(CTMM)
+periodic.svf <- function(CTMM,...)
 {
   if(all(CTMM$harmonic==0)) { return( stationary.svf(CTMM) ) }
 
@@ -248,18 +417,8 @@ periodic.svf <- function(CTMM)
   return(list(EST=EST,VAR=VAR))
 }
 
-
-# name of mean function
-periodic.name <- function(CTMM)
-{
-  NAME <- CTMM$harmonic
-  NAME <- paste(NAME,collapse=" ")
-  NAME <- paste("harmonic",NAME)
-  return(NAME)
-}
-
 # calculate deterministic mean square speed and its variance
-periodic.speed <- function(CTMM)
+periodic.speed <- function(CTMM,...)
 {
   if(all(CTMM$harmonic==0)) { return(stationary.speed(CTMM)) }
 
@@ -299,15 +458,64 @@ periodic.energy <- function(CTMM)
   return(list(UU=UU,VV=VV))
 }
 
+# list of non-zero frequencies of model
+periodic.omega <- function(CTMM)
+{
+  harmonic <- CTMM$harmonic
+  period <- CTMM$period
+
+  omega <- NULL
+  for(i in 1%:%length(harmonic))
+  {
+    w <- (2*pi/period[i]) * (1%:%harmonic[i])
+    omega <- c( omega , w )
+  }
+
+  return(omega)
+}
+
+# useful info from periodic mean function parameters
+periodic.stuff <- function(CTMM)
+{
+  AXES <- length(CTMM$axes)
+  harmonic <- CTMM$harmonic
+  period <- CTMM$period
+  omega <- periodic.omega(CTMM)
+
+  # amplitudes and covariance
+  A <- CTMM$mu
+  COV <- CTMM$COV.mu
+
+  # total number of harmonics
+  k <- length(omega)
+
+  # default uncertainty of none
+  if(is.null(COV)) { COV <- 0 }
+  # default structure
+  COV <- array(COV,c(AXES,2*k+1,2*k+1,AXES))
+
+  # delete stationary coefficients
+  A <- A[-1,,drop=FALSE]
+  COV <- COV[,-1,,,drop=FALSE]
+  COV <- COV[,,-1,,drop=FALSE]
+
+  # structure omega like A
+  omega <- viffle(omega,omega) # riffle(sin,cos)
+  omega <- cbind(omega,omega) # cbind(x,y)
+  # this is now (2k)*2
+
+  # flatten block-vectors and block-matrices
+  A <- array(A,(2*k)*AXES)
+  omega <- array(omega,(2*k)*AXES)
+  COV <- aperm(COV,c(2,1,3,4))
+  COV <- array(COV,c((2*k)*AXES,(2*k)*AXES))
+
+  return(list(A=A,COV=COV,omega=omega))
+}
 
 # calculate rotational indices
-periodic.summary <- function(CTMM,level,level.UD)
+periodic.variances <- function(CTMM,...)
 {
-  alpha <- 1-level
-  SUM <- NULL
-
-  if(all(CTMM$harmonic==0)) { return(SUM) }
-
   STUFF <- periodic.stuff(CTMM)
   omega <- STUFF$omega
   A <- STUFF$A
@@ -321,7 +529,7 @@ periodic.summary <- function(CTMM,level,level.UD)
   GRAD <- A
   COV.ROT <- c(GRAD %*% COV %*% GRAD)
 
-  if(CTMM$isotropic) { PARS <- c('major','major') } else { PARS <- c('major','minor') }
+  if(CTMM$isotropic[1]) { PARS <- c('major','major') } else { PARS <- c('major','minor') }
   COV.RAN <- CTMM$COV[PARS,PARS]
   GRAD <- c(1,1)
   COV.RAN <- c(GRAD %*% COV.RAN %*% GRAD)
@@ -329,11 +537,17 @@ periodic.summary <- function(CTMM,level,level.UD)
   GRAD <- c(RAN,-ROT)/(ROT+RAN)^2
   VAR <- sum(GRAD^2 * c(COV.ROT,COV.RAN))
 
-  CI <- ci.tau(MLE,VAR,alpha=alpha,min=0,max=1)
-  CI <- 100*sqrt(rbind(CI))
-  rownames(CI) <- "rotation/deviation %"
+  MLE <- ROT/(ROT+RAN)
+  GRAD <- c(RAN,-ROT)/(ROT+RAN)^2
+  VAR <- sum(GRAD^2 * c(COV.ROT,COV.RAN))
 
-  SUM <- rbind(SUM,CI)
+  R <- list()
+  R$MLE <- MLE
+  R$VAR <- VAR
+
+  V <- list()
+  V$MLE <- 0
+  V$VAR <- Inf
 
   # ROTATIONAL SPEED INDEX
   # CIs are too big to be useful
@@ -354,143 +568,91 @@ periodic.summary <- function(CTMM,level,level.UD)
     GRAD <- c(RAN,-ROT)/(ROT+RAN)^2
     VAR <- sum(GRAD^2 * c(COV.ROT,COV.RAN))
 
-    CI <- ci.tau(MLE,VAR,alpha=alpha,min=0,max=1)
-    CI <- 100*sqrt(rbind(CI))
-    rownames(CI) <- "rotation/speed %"
-
-    SUM <- rbind(SUM,CI)
+    V <- list()
+    V$MLE <- MLE
+    V$VAR <- VAR
   }
 
-  return(SUM)
+  return(list(R=R,V=V))
 }
 
-# increase number of harmonics in model
-periodic.drift.complexify <- function(CTMM)
+
+# are the non-linear parameters finite?
+periodic.is.finite <- function(CTMM,data,...)
 {
-  period <- CTMM$period
-  harmonic <- CTMM$harmonic
+  period <- max(CTMM$period)
+  dt <- last(data$t)-first(data$t)
 
-  GUESS <- list()
+  if(period > 2*pi*dt * 10)
+  { R <- FALSE }
+  else
+  { R <- TRUE }
 
-  # dt <- stats::median(diff(data$t))
-  # P <- attr(CTMM$mean,"par")$P
-  # # max harmonics for Nyquist frequency
-  # kmax <- P/(2*dt)
-
-  for(i in 1:length(period))
-  {
-    GUESS[[length(GUESS)+1]] <- CTMM
-    GUESS[[length(GUESS)]]$harmonic[i] <- harmonic[i] + 1
-  }
-
-  return(GUESS)
+  return(R)
 }
 
-# reduce number of harmonics in model
-periodic.drift.simplify <- function(CTMM)
-{
-  period <- CTMM$period
-  harmonic <- CTMM$harmonic
-
-  GUESS <- list()
-
-  for(i in 1:length(period))
-  {
-    if(harmonic[i]>0)
-    {
-      GUESS[[length(GUESS)+1]] <- CTMM
-      GUESS[[length(GUESS)]]$harmonic[i] <- harmonic[i] - 1
-    }
-  }
-
-  return(GUESS)
-}
-
-# list of non-zero frequencies of model
-periodic.omega <- function(CTMM)
-{
-  harmonic <- CTMM$harmonic
-  period <- CTMM$period
-
-  omega <- NULL
-  for(i in 1:length(harmonic))
-  {
-    if(harmonic[i] > 0)
-    {
-      w <- (2*pi/period[i]) * (1:harmonic[i])
-      omega <- c( omega , w )
-    }
-  }
-
-  return(omega)
-}
-
-# useful info from periodic mean function parameters
-periodic.stuff <- function(CTMM)
-{
-  harmonic <- CTMM$harmonic
-  period <- CTMM$period
-  omega <- periodic.omega(CTMM)
-
-  # amplitudes and covariance
-  A <- CTMM$mu
-  COV <- CTMM$COV.mu
-
-  # total number of harmonics
-  k <- length(omega)
-
-  # default uncertainty of none
-  if(is.null(COV)) { COV <- 0 }
-  # default structure
-  COV <- array(COV,c(2,2*k+1,2*k+1,2))
-
-  # delete stationary coefficients
-  A <- A[-1,,drop=FALSE]
-  COV <- COV[,-1,,,drop=FALSE]
-  COV <- COV[,,-1,,drop=FALSE]
-
-  # structure omega like A
-  omega <- c(omega,omega)
-  omega <- cbind(omega,omega)
-  # this is now (2k)*2
-
-  # flatten block-vectors and block-matrices
-  A <- array(A,(2*k)*2)
-  omega <- array(omega,(2*k)*2)
-  COV <- aperm(COV,c(2,1,3,4))
-  COV <- array(COV,c((2*k)*2,(2*k)*2))
-
-  return(list(A=A,COV=COV,omega=omega))
-}
-
-# combine this all together for convenience
-periodic <- new.drift(periodic.drift,
-                      is.stationary=periodic.is.stationary,
-                      energy=periodic.energy,
-                      init=periodic.init,
-                      name=periodic.name,
-                      refine=periodic.drift.complexify,
-                      scale=periodic.scale,
-                      speed=periodic.speed,
-                      summary=periodic.summary,
-                      svf=periodic.svf,
-                      velocity=periodic.velocity)
-
 ##################################
 # change-point mean functions
 ##################################
 # change.point slot is a data.frame with columns: start, stop, state
 
-change.point.is.stationary <- function(CTMM)
+# name of mean function
+change.point.name <- function(CTMM,...)
+{
+  CP <- CTMM$change.point.mu # change points
+  if(is.null(CP)) { CP <- CTMM$change.point } # default
+
+  NAME <- NULL
+  for(s in levels(CP$state))
+  {
+    M <- CTMM[[s]]
+    N <- get(M$mean)$name(M)
+    if(length(N)) { NAME <- paste0(NAME,N,sep="-") }
+  }
+
+  return(NAME)
+}
+
+change.point.is.stationary <- function(CTMM,...)
 {
   CP <- CTMM$change.point.mu # change points
   if(is.null(CP)) { CP <- CTMM$change.point } # default
   nlevels(CP$state)==1
 }
 
+# guess parameters
+change.point.init <- function(CTMM,data=NULL,...)
+{
+  CP <- CTMM$change.point.mu # change points
+  if(is.null(CP)) { CP <- CTMM$change.point } # default
+
+  # get times for each state
+  SUBS <- list()
+  i1 <- i2 <- 1
+  for(j in 1:nrow(CP))
+  {
+    while(t[i2+1]<CP$stop[j]) { i2 <- i2+1 }
+    SUB <- i1:i2
+
+    s <- as.character(CP$state[j])
+    SUBS[[s]] <- c(SUBS[[2]],SUB)
+
+    i1 <- i2 <- i2+1
+  }
+
+  for(s in levels(CP$state))
+  {
+    SUB <- data[SUBS[[s]],]
+    M <- CTMM[[s]]
+    CTMM[[s]] <- get(M$mean)$init(SUB,M)
+  }
+
+  return(CTMM)
+}
+
 
 # periodic mean/drift function
-change.point.drift <- function(t,CTMM,velocity=FALSE)
+change.point.mean <- function(CTMM,t,velocity=FALSE,...)
 {
   CP <- CTMM$change.point.mu # change points
   if(is.null(CP)) { CP <- CTMM$change.point } # default
@@ -534,56 +696,63 @@ change.point.drift <- function(t,CTMM,velocity=FALSE)
 }
 
 # periodic velocity mean
-change.point.velocity <- function(t,CTMM)
-{ change.point.drift(t=t,CTMM=CTMM,velocity=TRUE) }
+change.point.velocity <- function(CTMM,t,...)
+{ change.point.mean(t=t,CTMM=CTMM,velocity=TRUE) }
 
-# guess parameters
-change.point.init <- function(data,CTMM)
+# increase number of harmonics in model
+change.point.complexify <- function(CTMM,simplify=FALSE)
 {
   CP <- CTMM$change.point.mu # change points
   if(is.null(CP)) { CP <- CTMM$change.point } # default
 
-  # get times for each state
-  SUBS <- list()
-  i1 <- i2 <- 1
-  for(j in 1:nrow(CP))
-  {
-    while(t[i2+1]<CP$stop[j]) { i2 <- i2+1 }
-    SUB <- i1:i2
-
-    s <- as.character(CP$state[j])
-    SUBS[[s]] <- c(SUBS[[2]],SUB)
-
-    i1 <- i2 <- i2+1
-  }
-
+  GUESS <- list()
   for(s in levels(CP$state))
   {
-    SUB <- data[SUBS[[s]],]
     M <- CTMM[[s]]
-    CTMM[[s]] <- get(M$mean)$init(SUB,M)
+    if(!simplify)
+    { GUESS <- c(GUESS, get(M$mean)$complexify(M) ) }
+    else
+    { GUESS <- c(GUESS, get(M$mean)$simplify(M) ) }
   }
 
+  if(!length(GUESS)) { GUESS <- NULL }
+  return(GUESS)
+}
+
+# reduce number of harmonics in model
+change.point.simplify <- function(CTMM)
+{ change.point.complexify(CTMM,simplify=TRUE) }
+
+# how do we rescale time
+change.point.scale <- function(CTMM,time,...)
+{
+  CP <- CTMM$change.point.mu # change points
+  if(is.null(CP)) { CP <- CTMM$change.point } # default
+
+  for(s in levels(CP$state))
+  { CTMM[[s]] <- drift.scale(CTMM[[s]],time) }
+
   return(CTMM)
 }
 
-# how do we rescale time
-change.point.scale <- function(CTMM,time)
+# calculate rotational indices
+change.point.summary <- function(CTMM,level,level.UD,...)
 {
   CP <- CTMM$change.point.mu # change points
   if(is.null(CP)) { CP <- CTMM$change.point } # default
 
+  CI <- NULL
   for(s in levels(CP$state))
   {
     M <- CTMM[[s]]
-    CTMM[[s]] <- get(M$mean)$scale(M,time)
+    CI <- rbind(CI, get(M$mean)$summary(M,level,level.UD) )
   }
 
-  return(CTMM)
+  return(CI)
 }
 
 # SVF of mean function
-change.point.svf <- function(CTMM,speed=FALSE)
+change.point.svf <- function(CTMM,speed=FALSE,...)
 {
   CP <- CTMM$change.point.mu # change points
   if(is.null(CP)) { CP <- CTMM$change.point } # default
@@ -615,30 +784,12 @@ change.point.svf <- function(CTMM,speed=FALSE)
   return(list(EST=EST,VAR=VAR))
 }
 
-
-# name of mean function
-change.point.name <- function(CTMM)
-{
-  CP <- CTMM$change.point.mu # change points
-  if(is.null(CP)) { CP <- CTMM$change.point } # default
-
-  NAME <- NULL
-  for(s in levels(CP$state))
-  {
-    M <- CTMM[[s]]
-    N <- get(M$mean)$name(M)
-    if(length(N)) { NAME <- paste0(NAME,N,sep="-") }
-  }
-
-  return(NAME)
-}
-
 # calculate deterministic mean square speed and its variance
-change.point.speed <- function(CTMM)
+change.point.speed <- function(CTMM,...)
 { change.point.svf(CTMM,speed=TRUE) }
 
 # UU and VV terms
-change.point.energy <- function(CTMM)
+change.point.energy <- function(CTMM,...)
 {
   CP <- CTMM$change.point.mu # change points
   if(is.null(CP)) { CP <- CTMM$change.point } # default
@@ -671,67 +822,37 @@ change.point.energy <- function(CTMM)
   return(list(UU=UU,VV=VV))
 }
 
+#################################
+# continuous uniform spline mean functions (UNFINISHED)
+#################################
 
-# calculate rotational indices
-change.point.summary <- function(CTMM,level,level.UD)
+# name of mean function
+uspline.name <- function(CTMM,...)
 {
-  CP <- CTMM$change.point.mu # change points
-  if(is.null(CP)) { CP <- CTMM$change.point } # default
-
-  CI <- NULL
-  for(s in levels(CP$state))
-  {
-    M <- CTMM[[s]]
-    CI <- rbind(CI, get(M$mean)$summary(M,level,level.UD) )
-  }
-
-  return(CI)
+  NAME <- paste("degree",CTMM$degree,"knot",CTMM$knot)
+  return(NAME)
 }
 
-# increase number of harmonics in model
-change.point.drift.complexify <- function(CTMM,simplify=FALSE)
+# not stationary if knots
+uspline.is.stationary <- function(CTMM,...) { !sum(CTMM$knot) }
+
+# initialize default parameters
+uspline.init <- function(CTMM,data=NULL,...)
 {
-  CP <- CTMM$change.point.mu # change points
-  if(is.null(CP)) { CP <- CTMM$change.point } # default
+  # degree of continuity: 1,2 - location,velocity
+  if(is.null(CTMM$degree)) { CTMM$degree <- 1 }
+  # default number of knots
+  if(is.null(CTMM$knot)) { CTMM$knot <- 1 }
+  # default domain of spline grid
+  if(is.null(CTMM$domain)) { CTMM$domain <- c(data$t[1],last(data$t)) }
 
-  GUESS <- list()
-  for(s in levels(CP$state))
-  {
-    M <- CTMM[[s]]
-    if(!simplify)
-    { GUESS <- c(GUESS, get(M$mean)$complexify(M) ) }
-    else
-    { GUESS <- c(GUESS, get(M$mean)$simplify(M) ) }
-  }
+  if(is.null(CTMM$mu)) { CTMM <- drift.init(CTMM,data,...) }
 
-  if(!length(GUESS)) { GUESS <- NULL }
-  return(GUESS)
+  return(CTMM)
 }
 
-# reduce number of harmonics in model
-change.point.drift.simplify <- function(CTMM)
-{ change.point.drift.complexify(CTMM,simplify=TRUE) }
-
-# combine this all together for convenience
-change.point <- new.drift(change.point.drift,
-                          is.stationary=change.point.is.stationary,
-                          energy=change.point.energy,
-                          init=change.point.init,
-                          name=change.point.name,
-                          refine=change.point.drift.complexify,
-                          scale=change.point.scale,
-                          speed=change.point.speed,
-                          summary=change.point.summary,
-                          svf=change.point.svf,
-                          velocity=change.point.velocity)
-
-
-#################################
-# continuous uniform spline mean functions
-#################################
-uspline.is.stationary <- function(CTMM) { !sum(CTMM$knot) }
-
-uspline.drift <- function(t,CTMM)
+# mean function
+uspline.mean <- function(CTMM,t,...)
 {
   degree <- CTMM$degree
   knot <- CTMM$knot
@@ -743,7 +864,7 @@ uspline.drift <- function(t,CTMM)
   # midpoint / no real grid / degenerate case
   if(knot==1)
   {
-    U <- stationary.drift(t,CTMM)
+    U <- stationary.mean(CTMM,t,...)
     if(degree==1) { return( U ) }
 
     U <- cbind( U , (t-tknot) )
@@ -782,23 +903,15 @@ uspline.drift <- function(t,CTMM)
   return(U)
 }
 
-# initialize default parameters
-uspline.init <- function(data,CTMM)
+# build-up mean function
+uspline.complexify <- function(CTMM)
 {
-  # degree of continuity: 1,2 - location,velocity
-  if(is.null(CTMM$degree)) { CTMM$degree <- 1 }
-  # default number of knots
-  if(is.null(CTMM$knot)) { CTMM$knot <- 1 }
-  # default domain of spline grid
-  if(is.null(CTMM$domain)) { CTMM$domain <- c(data$t[1],last(data$t)) }
-
-  if(is.null(CTMM$mu)) { CTMM <- drift.init(data,CTMM) }
-
-  return(CTMM)
+  CTMM$knot <- CTMM$knot + 1
+  return(list(CTMM))
 }
 
 # scale times
-uspline.scale <- function(CTMM,time)
+uspline.scale <- function(CTMM,time,...)
 {
   knot <- CTMM$knot
   degree <- CTMM$degree
@@ -816,25 +929,16 @@ uspline.scale <- function(CTMM,time)
   return(CTMM)
 }
 
+# summary
+#
+#
+
 # svf
-
-# refine
-uspline.refine <- function(CTMM)
-{
-  CTMM$knot <- CTMM$knot + 1
-  return(list(CTMM))
-}
-
-
-# name of mean function
-uspline.name <- function(CTMM)
-{
-  NAME <- paste("degree",CTMM$degree,"knot",CTMM$knot)
-  return(NAME)
-}
+#
+#
 
 # ms speed of spline function
-uspline.speed <- function(CTMM)
+uspline.speed <- function(CTMM,...)
 {
   degree <- CTMM$degree
   knot <- CTMM$knot
@@ -861,10 +965,12 @@ uspline.speed <- function(CTMM)
   return(list(EST=EST,VAR=VAR))
 }
 
-# summary
+# energy
+#
+#
 
 # calculate spline grid
-uspline.stuff <- function(CTMM)
+uspline.stuff <- function(CTMM,...)
 {
   knot <- CTMM$knot
   domain <- CTMM$domain
@@ -885,21 +991,13 @@ uspline.stuff <- function(CTMM)
   return(list(tknot=tknot,dt=dt))
 }
 
-# convenience function
-uspline <- new.drift(uspline.drift,
-                     is.stationary=uspline.is.stationary,
-                     init=uspline.init,
-                     scale=uspline.scale,
-                     shift=uniform.shift,
-                     refine=uspline.refine,
-                     name=uspline.name)
 
 #################################
-# piecewise-stationary mean/drift function
+# piecewise-stationary mean/drift function (UNFINISHED)
 #################################
 pwstationary.is.stationary <- function(CTMM) { !length(CTMM$breaks) }
 
-pwstationary.drift <- function(t,CTMM)
+pwstationary.mean <- function(CTMM,t,...)
 {
   breaks <- CTMM$breaks
   id <- factor(breaks$id)
@@ -936,19 +1034,14 @@ pwstationary.drift <- function(t,CTMM)
   return(u)
 }
 
-############ init handle posixct start & stop
-
-#############
-pwstationary.scale <- function(CTMM,time)
+# rescale time
+pwstationary.scale <- function(CTMM,time,...)
 {
   CTMM$breaks[,c('start','stop')] <- CTMM$breaks[,c('start','stop')] / time
 
   return(CTMM)
 }
 
-# combine this all together for convenience
-pwstationary <- new.drift(pwstationary.drift,
-                          is.stationary=pwstationary.is.stationary,
-                          shift=uniform.shift,
-                          speed=stationary.speed,
-                          svf=stationary.svf)
+#################################
+
+
