@@ -248,7 +248,6 @@ ctmm.iterate <- function(data,CTMM,verbose=FALSE,level=1,IC="AICc",MSPE="positio
     for(i in 1:AXES) { if(CTMM$sigma@par[i]>.Machine$double.eps) { VAR[i] <<- CTMM$sigma@par[i] } }
   }
 
-  drift <- get(CTMM$mean)
   if(CTMM$mean=="periodic")
   {
     Nyquist <- CTMM$period/stats::median(diff(data$t))/2
@@ -271,7 +270,7 @@ ctmm.iterate <- function(data,CTMM,verbose=FALSE,level=1,IC="AICc",MSPE="positio
     REFINE <- REFINE[!(names(REFINE) %in% TRYS)]
 
     # copy over best initial parameter guess for refined drops
-    if(!drift@is.stationary(CTMM) && length(DROP))
+    if(!drift.is.stationary(CTMM) && length(DROP))
     {
       for(i in 1:length(DROP))
       {
@@ -423,13 +422,13 @@ ctmm.iterate <- function(data,CTMM,verbose=FALSE,level=1,IC="AICc",MSPE="positio
       if("tau velocity" %in% VARS) # OUF -> OU / IOU -> BM
       {
         Q <- CI["tau velocity",1]
-        if(is.na(Q) || (Q<=0))
+        if(level==1 || is.na(Q) || (Q<=0))
         { GUESS <- c(GUESS,list(simplify.ctmm(MLE,'tau velocity'))) }
       }
       else if("omega" %nin% VARS) # OUf -> OU, IID
       {
         Q <- CI["tau",1]
-        if(is.na(Q) || (Q<=0))
+        if(level==1 || is.na(Q) || (Q<=0))
         {
           GUESS <- c(GUESS,list(simplify.ctmm(MLE,'tau'))) # OUf -> IID
           GUESS <- c(GUESS,list(simplify.ctmm(MLE,'tau velocity'))) # OUf -> OU
@@ -438,14 +437,14 @@ ctmm.iterate <- function(data,CTMM,verbose=FALSE,level=1,IC="AICc",MSPE="positio
       else # OUO -> OUf
       {
         Q <- 1/CI["tau period",3]
-        if(is.na(Q) || (Q<=0))
+        if(level==1 || is.na(Q) || (Q<=0))
         { GUESS <- c(GUESS,list(simplify.ctmm(MLE,'omega'))) }
       }
     } # end # OUX -> OUx
     else if(CTMM$range && length(CTMM$tau)==1 && !is.na(IC)) # OU -> IID
     {
       Q <- CI["tau position",1]
-      if(is.na(Q) || (Q<=0))
+      if(level==1 || is.na(Q) || (Q<=0))
       { GUESS <- c(GUESS,list(simplify.ctmm(MLE,'tau position'))) }
     } # end # OU -> IID
 
@@ -459,7 +458,7 @@ ctmm.iterate <- function(data,CTMM,verbose=FALSE,level=1,IC="AICc",MSPE="positio
       Q <- c(J %*% CTMM$COV[Q,Q] %*% J) # variance of nu
       Q <- ci.tau(nu,Q,alpha=beta)[1]
 
-      if(is.na(Q) || Q<=0 || level==1 || is.na(IC))
+      if(level==1 || is.na(Q) || Q<=0 || is.na(IC))
       { GUESS <- c(GUESS,list(simplify.ctmm(MLE,"diff.tau"))) }
     }
     else if("tau" %in% VARS) # try other side if boundary if chosen model is critically damped
@@ -481,7 +480,7 @@ ctmm.iterate <- function(data,CTMM,verbose=FALSE,level=1,IC="AICc",MSPE="positio
     if("circle" %in% VARS)
     {
       Q <- CI["circle",3]
-      if(is.na(Q) || (Q==Inf) || is.na(IC))
+      if(level==1 || is.na(Q) || (Q==Inf) || is.na(IC))
       { GUESS <- c(GUESS,list(simplify.ctmm(MLE,"circle"))) }
     }
 
@@ -493,7 +492,7 @@ ctmm.iterate <- function(data,CTMM,verbose=FALSE,level=1,IC="AICc",MSPE="positio
       SD <- ifelse(all(Q %in% CTMM$features),sqrt(c(GRAD %*% CTMM$COV[Q,Q] %*% GRAD)),Inf) # variance could collapse early
       Q <- log(CTMM$sigma@par['major']/CTMM$sigma@par['minor'])
       Q <- stats::qnorm(beta/2,mean=Q,sd=SD)
-      if(is.na(Q) || Q<=0 || is.na(IC))
+      if(level==1 || is.na(Q) || Q<=0 || is.na(IC))
       { GUESS <- c(GUESS,list(simplify.ctmm(MLE,"minor"))) }
     }
 
@@ -611,11 +610,10 @@ name.ctmm <- function(CTMM,whole=TRUE)
 
   # data link
   link <- get.link(CTMM)
-  if(link$name!="identity") { NAME <- c(NAME,paste0(link$name,"-link")) }
+  if(link$name!="identity") { NAME <- c(NAME,link$name) }
 
   # mean
-  drift <- get(CTMM$mean)
-  DNAME <- drift@name(CTMM)
+  DNAME <- drift.name(CTMM)
 
   NAME <- paste(NAME,sep="",collapse=" ")
 
